#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import numpy as np
import matplotlib.pyplot as plt

def main():
    m = 100
    n = 100
    x = 2*(np.arange(0,m) / (m-1.) -0.5)

    Z = np.zeros((m,n))

    for i in range(0,n):
        Z[:,i] = x**i

    U, s, V = np.linalg.svd(Z, full_matrices=True)

    plt.subplot(1,3,1)
    for i in range(0,n):
        plt.plot(x,Z[:,i])

    plt.subplot(1,3,2)
    plt.semilogy(list(range(1,n+1)),s,'*')
    plt.xlabel('i')
    plt.ylabel('sigma_i')

    print(V.shape)
    print(Z.shape)
    plt.subplot(1,3,3)

    ZV = np.dot(Z,np.transpose(V))
    for i in range(0,n):
        plt.plot(x,ZV[:,i])

    plt.show()
if __name__ == "__main__":
    main()
