#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.



import matplotlib as mpl
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import LinearLocator, FormatStrFormatter
#from func import grid

def grid(M):

    M = M.astype(int)
    TMP = M.shape

    d = TMP[0]

    m_max = 0
    for i in range(0,d):
        if m_max < M[i]:
            m_max = M[i]

    X = np.zeros((m_max,d))
    W = np.zeros((m_max,d))

    for i in range(0,d):
        x, w = np.polynomial.legendre.leggauss(M[i])
        for j in range(0,M[i]):
            X[j,i] = x[j]
            W[j,i] = w[j]
    return X, W

def main():
    m = 20
    n = 20

    M = np.zeros((2,))
    M[0] = m
    M[1] = m
    X_0, W_0 = grid(M)

    X = np.zeros((m,m))
    Y = np.zeros((m,m))

    Z_1 = np.zeros((m,n))
    Z_2 = np.zeros((m**2,n**2))

    X, Y = np.meshgrid(X_0[:,0], X_0[:,1])

    x_2 = X
    y_2 = Y

    x_2 = np.reshape(x_2,(m**2,))
    y_2 = np.reshape(y_2,(m**2,))

    for i in range(0,n):
        Z_1[:,i] = X_0[:,0]**i

    W_1 = np.zeros((m,m))
    W_1[:m,:m] = np.diag(W_0[:,0])

    W_1_sr = np.sqrt(W_1)

    W_1_srZ_1 = np.dot(W_1_sr,Z_1)

    for i in range(0,n):
        for j in range(0,n):
            Z_2[:,j+n*i] = x_2**i*y_2**j

    W_2 = np.zeros((m**2,m**2))

    tmp = np.zeros((m**2,))
    for i in range(0,m):
        for j in range(0,m):
            tmp[j+m*i] = W_0[i,0]*W_0[j,1]

    W_2[:m**2,:m**2] = np.diag(tmp)

    W_2_sr = np.sqrt(W_2)

    W_2_srZ_2 = np.dot(W_2_sr,Z_2)

    U_1, s_1, V_1 = np.linalg.svd(W_1_srZ_1, full_matrices=True)
    U_2, s_2, V_2 = np.linalg.svd(W_2_srZ_2, full_matrices=True)

    plt.figure()

    plt.semilogy(list(range(1,n+1)),s_1**2,'ro')
    plt.semilogy(list(range(1,n**2+1)),s_2,'ko')
    for i in range(0,n):
        plt.semilogy([1,n**2+1],[s_1[i]**2,s_1[i]**2],'b')

    fig = plt.figure()
    ax = fig.gca(projection='3d')

    Z_2V_2 = np.dot(Z_2,np.transpose(V_2))

    phi_1 = np.reshape(Z_2V_2[:,0],(m,m)) 
    phi_3 = np.reshape(Z_2V_2[:,2],(m,m)) 
    phi_5 = np.reshape(Z_2V_2[:,4],(m,m)) 

    surf = ax.plot_surface(X, Y, phi_1, rstride=1, cstride=1, cmap=cm.jet,
                       linewidth=0, antialiased=False)
    surf = ax.plot_surface(X, Y, phi_3, rstride=1, cstride=1, cmap=cm.jet,
                       linewidth=0, antialiased=False)
    surf = ax.plot_surface(X, Y, phi_5, rstride=1, cstride=1, cmap=cm.jet,
                       linewidth=0, antialiased=False)
    
    ax.axis('equal')
    plt.show()

if __name__ == "__main__":
    main()
